import os
import sys
import h5py
import numpy as np
import datatable as dt


def ims2csv(file_name, outfile=None):
	#read in image file
	
	file_name = os.path.abspath(file_name)
	f = h5py.File(file_name, 'r')
	#f.visit(print)
	#todo ensure that 'DataSet' and 'ResolutionLevel' each have only 1 key
	
	channel_names = list(f['DataSet']['ResolutionLevel 0']['TimePoint 0'].keys())
	
	
	Cn = [''.join(f['DataSetInfo'][ch].attrs['Name'].astype(str)) for ch in channel_names]
	Dn = [''.join(f['DataSetInfo'][ch].attrs['DyeName'].astype(str)) for ch in channel_names]
	Ex = [round(float(''.join(f['DataSetInfo'][ch].attrs['LSMExcitationWavelength'].astype(str)))) for ch in channel_names]
	Em = [round(float(''.join(f['DataSetInfo'][ch].attrs['LSMEmissionWavelength'].astype(str)))) for ch in channel_names]
	Ex = ['Ex' + str(x) for x in Ex]
	Em = ['Em' + str(x) for x in Em]
	for i in range(len(Dn)):
		if Dn[i] == '':
			Dn[i] = Ex[i] + '/' + Em[i]
	
	nice_names = [Cn + " [" + Dn + "]" for Cn, Dn in zip(Cn, Dn)]
	
	a = [f['DataSet']['ResolutionLevel 0']['TimePoint 0'][ch]['Data'] for ch in channel_names]
	
	a=np.stack(a)
	
	a=np.transpose(a)
	
	#flatten array
	
	Zaxis = a.shape[2]
	
	slicer = [np.any(a[:,:,i,:]) for i in range(Zaxis)]
	
	a = a[:,:,slicer,:]
	
	NI, NJ, NK, CH = a.shape
	
	dat = dt.Frame(x = np.repeat(range(NI),NJ*NK), 
	y = np.tile(np.repeat(range(NJ),NK), NI),
	z = np.tile(range(NK), NI*NJ))
	
	
	for ch in range(CH):
		dat[channel_names[ch]] = np.float32(a[:,:,:,ch].flatten())
	del dat.names
	dat.names = ['x', 'y', 'z'] + nice_names
	#save as a csv file
	
	if outfile is None:
		file_name = os.path.basename(file_name)
		base = os.path.splitext(file_name)[0]
		
		new_file_name = base + '.csv'
		
		current_dir = os.getcwd()
		
		new_file_name = os.path.join(current_dir, new_file_name)
	else: new_file_name=outfile
		
	
	dat.to_csv(new_file_name)
	print('file saved to ' + new_file_name)








#!/usr/local/bin/python3

import os
import sys
import argparse
import h5py
import numpy as np 
import datatable as dt
import imaris_tools
#exec(open("/Users/kevin/Desktop/imaris_tools/imaris_tools.py").read())

class ArgumentParserError(Exception):
	pass

class NewArgumentParser(argparse.ArgumentParser):
	def error(self, message):
		print(message)
		sys.exit(0)


def parse_args(args):
	p = NewArgumentParser(description='convert an ims file to a csv file')
	a = p.add_argument_group('data loading parameters')
	a.add_argument('filename',
					help='File path of input data ims file.')
	a.add_argument('-o', '--output_file', metavar='O', required=False, default = None,
					help='File path of the output csv file. default: a file saved to the present working directory where the .ims extension is replaced by .csv')
	try:
		return p.parse_args(args)
	except ArgumentParserError:
		raise


def main(args: list = None):
	args = parse_args(args)
	print(args)
	try:
		imaris_tools.ims2csv(file_name=args.filename, outfile=args.output_file)

	except:
		raise

if __name__ == '__main__':
	main(sys.argv[1:])



import os
import sys
import shutil
from setuptools import setup
from warnings import warn

if sys.version_info.major != 3:
    raise RuntimeError('imaris_tools requires Python 3')


setup(name='imaris_tools',
      version='0.1.4',
      description='manipulation of ims files',
      author='',
      author_email='',
      package_dir={'': 'src'},
      packages=['imaris_tools'],
      install_requires=[
          'numpy',
          'datatable',
          'h5py',
          ],
      scripts=['src/imaris_tools/ims2csv.py'],
      )


# get location of setup.py
setup_dir = os.path.dirname(os.path.realpath(__file__))
